import React, { useState, useRef, useEffect } from 'react';
import { Container, Button } from 'react-bootstrap';
import { Dot, ChevronDown, ChevronUp, SlashCircle, CaretUpFill, CaretUp, CaretDownFill, CaretDown, FilePdf, Image, FilePlay, ArrowRight, HandThumbsUpFill, HandThumbsUp, HandThumbsDownFill, HandThumbsDown, X, EmojiSmile, Keyboard, Paperclip, Send, FileImage, Trash, PlayBtn, CardImage } from 'react-bootstrap-icons';
import 'react-ui-components-superflows';
import { UploadToS3 } from 'react-upload-to-s3';
import Themes from 'react-ui-themes-superflows';
import Picker from 'emoji-picker-react';

var Constants = {
  MODE_VIEW: "view",
  MODE_EDIT: "edit",
  MODE_DELETED: "deleted",
  FLOW_INIT: 0,
  FLOW_UPLOAD: 1,
  FLOW_UPLOAD_COMPLETE: 2,
  FLOW_EMOJI_PICKER: 3,
  FLOW_VIEW: 4,
  FLOW_DELETED: 5,
  FLOW_CONFIRM_DELETE: 6,
  FLOW_CONFIRM_REMOVE_LIKE: 7,
  FLOW_CONFIRM_REMOVE_DISLIKE: 8,
  FLOW_CONFIRM_REMOVE_UPVOTE: 9,
  FLOW_CONFIRM_REMOVE_DOWNVOTE: 10,
  FLOW_SHOW_EDIT: 11,
  FLOW_SHOW_ATTACHMENT: 12,
  UPLOAD_TYPE_IMAGE: "image",
  UPLOAD_TYPE_PDF: "pdf",
  UPLOAD_TYPE_VIDEO: "video",
  CONTENT_DELETED: "This content was deleted"
};

var Util = {
  ellipsizeStart: function ellipsizeStart(str, maxLength) {
    if (str.length > maxLength) {
      return "..." + str.slice(-(maxLength - 3));
    }

    return str;
  },
  getWindowDimensions: function getWindowDimensions() {
    var _window = window,
        width = _window.innerWidth,
        height = _window.innerHeight;
    return {
      width: width,
      height: height
    };
  },
  timeDifference: function timeDifference(current, previous) {
    var msPerMinute = 60 * 1000;
    var msPerHour = msPerMinute * 60;
    var msPerDay = msPerHour * 24;
    var msPerMonth = msPerDay * 30;
    var msPerYear = msPerDay * 365;
    var elapsed = current - previous;

    if (elapsed < msPerMinute) {
      return Math.round(elapsed / 1000) + ' seconds ago';
    } else if (elapsed < msPerHour) {
      return Math.round(elapsed / msPerMinute) + ' minutes ago';
    } else if (elapsed < msPerDay) {
      return Math.round(elapsed / msPerHour) + ' hours ago';
    } else if (elapsed < msPerMonth) {
      return Math.round(elapsed / msPerDay) + ' days ago';
    } else if (elapsed < msPerYear) {
      return Math.round(elapsed / msPerMonth) + ' months ago';
    } else {
      return Math.round(elapsed / msPerYear) + ' years ago';
    }
  }
};

var CommentView = function CommentView(props) {
  var _props$preFill;

  var _useState = useState(Util.getWindowDimensions());

  var _useState2 = useState(Constants.UPLOAD_TYPE_IMAGE),
      uploadType = _useState2[0],
      setUploadType = _useState2[1];

  var _useState3 = useState(''),
      uploadResult = _useState3[0],
      setUploadResult = _useState3[1];

  var _useState4 = useState(false),
      dirty = _useState4[0],
      setDirty = _useState4[1];

  var _useState5 = useState(false),
      showEmoji = _useState5[0],
      setShowEmoji = _useState5[1];

  var _useState6 = useState(false),
      showAttachments = _useState6[0],
      setShowAttachments = _useState6[1];

  var _useState7 = useState(false),
      showUpload = _useState7[0],
      setShowUpload = _useState7[1];

  var _useState8 = useState(Constants.MODE_VIEW),
      mode = _useState8[0],
      setMode = _useState8[1];

  var _useState9 = useState(Constants.FLOW_VIEW),
      flow = _useState9[0],
      setFlow = _useState9[1];

  var _useState10 = useState('{}'),
      replyTo = _useState10[0],
      setReplyTo = _useState10[1];

  var _useState11 = useState(false),
      edited = _useState11[0],
      setEdited = _useState11[1];

  var _useState12 = useState('{}'),
      preFill = _useState12[0],
      setPreFill = _useState12[1];

  var _useState13 = useState(''),
      textNew = _useState13[0],
      setTextNew = _useState13[1];

  var _useState14 = useState(''),
      textOriginal = _useState14[0],
      setTextOriginal = _useState14[1];

  var _useState15 = useState(true),
      disableSubmit = _useState15[0],
      setDisableSubmit = _useState15[1];

  var _useState16 = useState(null),
      deleteTimeoutHandler = _useState16[0],
      setDeleteTimeoutHandler = _useState16[1];

  var _useState17 = useState(null),
      removeLikeTimeoutHandler = _useState17[0],
      setRemoveLikeTimeoutHandler = _useState17[1];

  var _useState18 = useState(null),
      removeDisLikeTimeoutHandler = _useState18[0],
      setRemoveDisLikeTimeoutHandler = _useState18[1];

  var _useState19 = useState(null),
      removeUpVoteTimeoutHandler = _useState19[0],
      setRemoveUpVoteTimeoutHandler = _useState19[1];

  var _useState20 = useState(null),
      removeDownVoteTimeoutHandler = _useState20[0],
      setRemoveDownVoteTimeoutHandler = _useState20[1];

  var _useState21 = useState(true),
      likes = _useState21[0],
      setLikes = _useState21[1];

  var _useState22 = useState(true),
      disLikes = _useState22[0],
      setDisLikes = _useState22[1];

  var _useState23 = useState(false),
      iHaveLiked = _useState23[0],
      setIHaveLiked = _useState23[1];

  var _useState24 = useState(false),
      iHaveDisLiked = _useState24[0],
      setIHaveDisLiked = _useState24[1];

  var _useState25 = useState(true),
      upVotes = _useState25[0],
      setUpVotes = _useState25[1];

  var _useState26 = useState(true),
      downVotes = _useState26[0],
      setDownVotes = _useState26[1];

  var _useState27 = useState(false),
      iHaveUpVoted = _useState27[0],
      setIHaveUpVoted = _useState27[1];

  var _useState28 = useState(false),
      iHaveDownVoted = _useState28[0],
      setIHaveDownVoted = _useState28[1];

  var _useState29 = useState(false),
      callbackInfo = _useState29[0],
      setCallbackInfo = _useState29[1];

  var refInputNew = useRef(null);
  var refPopup = useRef(null);
  var theme = Themes.getTheme("Default");

  function setReplyToWrap(value) {
    setReplyTo(JSON.stringify(value));
  }

  function getReplyToWrap() {
    return JSON.parse(replyTo);
  }

  function setPreFillWrap(value) {
    setPreFill(JSON.stringify(value));
  }

  function getPreFillWrap() {
    return JSON.parse(preFill);
  }

  function setCallbackInfoWrap(value) {
    setCallbackInfo(JSON.stringify(value));
  }

  function getCallbackInfoWrap() {
    return JSON.parse(callbackInfo);
  }

  function showAttachment(value) {
    setShowAttachments(value);
  }

  function showEdit(value) {
    if (value) {
      setFlowWrap(Constants.FLOW_SHOW_EDIT);
    } else {
      setFlowWrap(Constants.FLOW_VIEW);
    }
  }

  function setFlowWrap(flow, timeout) {
    if (timeout === void 0) {
      timeout = 500;
    }

    setTimeout(function () {
      setFlow(flow);
    }, timeout);
  }

  function prepareUpload(type) {
    setUploadType(type);
    setTimeout(function () {
      setShowUpload(true);
    }, 500);
  }

  function cancelUpload() {
    setShowUpload(false);
  }

  function onUploadComplete(result) {
    setDirty(true);
    setFlowWrap(Constants.FLOW_UPLOAD_COMPLETE, 1500);

    if (uploadType == Constants.UPLOAD_TYPE_PDF || uploadType == Constants.UPLOAD_TYPE_IMAGE || uploadType == Constants.UPLOAD_TYPE_VIDEO) {
      setUploadResult(result.url);
    }
  }

  function onRemoveUpload() {
    setUploadResult('');
    setDirty(true);
    setFlowWrap(Constants.FLOW_INIT);
  }

  function onClickAttachment() {
    var url = '';
    var strArr = uploadResult.split("/");

    if (props.cdnPrefix == null) {
      url = 'https://' + props.bucket + ".s3." + props.awsRegion + ".amazonaws.com/" + strArr[1];
    } else {
      var lastCh = props.cdnPrefix;
      lastCh = lastCh.slice(-1);

      if (lastCh == "/") {
        url = props.cdnPrefix + strArr[1];
      } else {
        url = props.cdnPrefix + "/" + strArr[1];
      }
    }

    window.open(url);
  }

  function validateSubmit() {
    if (refInputNew.current != null) {
      if (refInputNew.current.value.length > 0) {
        setDisableSubmit(false);
        return;
      }
    }

    if (uploadResult.length > 0) {
      setDisableSubmit(false);
      return;
    }

    setDisableSubmit(true);
  }

  function submitResult() {
    setTextNew(refInputNew.current.value.replace(/\n$/, ""));
    if (props.onSubmit != null) props.onSubmit({
      text: refInputNew.current.value.replace(/\n$/, ""),
      attachment: uploadResult == '' ? null : {
        object: uploadResult,
        type: uploadType
      },
      user: props.user,
      callbackInfo: props.callbackInfo == null ? null : props.callbackInfo
    });

    if (props.preventEditToView != null) {
      if (props.preventEditToView) ; else {
        setMode(Constants.MODE_VIEW);
      }
    } else {
      setMode(Constants.MODE_VIEW);
    }

    if (props.clearOnSubmit) {
      setTextNew('');
      setTextOriginal('');
      setUploadResult('');
      refInputNew.current.value = '';
      setFlowWrap(Constants.FLOW_INIT);
    }
  }

  function onTextAreaKeyUp(event) {
    if (event.key == "Enter" && !event.shiftKey) {
      event.preventDefault();
      submitResult();
    } else {
      setTextNew(refInputNew.current.value);
      setDirty(true);
      validateSubmit();
    }
  }

  function onEmojiChosen(event, obj) {
    refInputNew.current.value = refInputNew.current.value + obj.emoji;
    setTextNew(refInputNew.current.value);
  }

  function openEmojiPicker(value) {
    setShowEmoji(value);
  }

  function onEditClicked() {
    setMode(Constants.MODE_EDIT);
  }

  function onCancelClicked() {
    setTextNew(textOriginal);
    setMode(Constants.MODE_VIEW);
  }

  function onDeleteClicked() {
    if (flow === Constants.FLOW_SHOW_EDIT) {
      setFlowWrap(Constants.FLOW_CONFIRM_DELETE);
    } else {
      if (props.onDelete != null) {
        props.onDelete(getCallbackInfoWrap());
      }

      setFlowWrap(Constants.FLOW_DELETED);
    }
  }

  function onReplyClicked() {
    if (props.onReplied != null) {
      props.onReplied(getCallbackInfoWrap());
    }
  }

  function onShareClicked() {
    if (props.onShared != null) {
      props.onShared(getCallbackInfoWrap());
    }
  }

  function onReplyToClosed() {
    if (props.onReplyToClosed != null) {
      props.onReplyToClosed(getCallbackInfoWrap());
    }
  }

  function onLiked() {
    if (!iHaveDisLiked) {
      if (props.onLiked != null) {
        props.onLiked(getCallbackInfoWrap());
      }

      setLikes(likes + 1);
      setIHaveLiked(true);
    }
  }

  function onDisLiked() {
    if (!iHaveLiked) {
      if (props.onDisLiked != null) {
        props.onDisLiked(getCallbackInfoWrap());
      }

      setDisLikes(disLikes + 1);
      setIHaveDisLiked(true);
    }
  }

  function onLikeRemoved() {
    if (flow === Constants.FLOW_VIEW) {
      setFlowWrap(Constants.FLOW_CONFIRM_REMOVE_LIKE);
    } else {
      if (props.onLikeRemoved != null) {
        props.onLikeRemoved(getCallbackInfoWrap());
      }

      setLikes(likes - 1);
      setIHaveLiked(false);
      setFlowWrap(Constants.FLOW_VIEW);
    }
  }

  function onDisLikeRemoved() {
    if (flow === Constants.FLOW_VIEW) {
      setFlowWrap(Constants.FLOW_CONFIRM_REMOVE_DISLIKE);
    } else {
      if (props.onDisLikeRemoved != null) {
        props.onDisLikeRemoved(getCallbackInfoWrap());
      }

      setDisLikes(disLikes - 1);
      setIHaveDisLiked(false);
      setFlowWrap(Constants.FLOW_VIEW);
    }
  }

  function onUpVoted() {
    if (!iHaveDownVoted) {
      if (props.onUpVoted != null) {
        props.onUpVoted(getCallbackInfoWrap());
      }

      setUpVotes(upVotes + 1);
      setIHaveUpVoted(true);
    }
  }

  function onDownVoted() {
    if (!iHaveUpVoted) {
      if (props.onDownVoted != null) {
        props.onDownVoted(getCallbackInfoWrap());
      }

      setDownVotes(downVotes + 1);
      setIHaveDownVoted(true);
    }
  }

  function onUpVoteRemoved() {
    if (flow === Constants.FLOW_VIEW) {
      setFlowWrap(Constants.FLOW_CONFIRM_REMOVE_UPVOTE);
    } else {
      if (props.onUpVoteRemoved != null) {
        props.onUpVoteRemoved(getCallbackInfoWrap());
      }

      setUpVotes(upVotes - 1);
      setIHaveUpVoted(false);
      setFlowWrap(Constants.FLOW_VIEW);
    }
  }

  function onDownVoteRemoved() {
    if (flow === Constants.FLOW_VIEW) {
      setFlowWrap(Constants.FLOW_CONFIRM_REMOVE_DOWNVOTE);
    } else {
      if (props.onDownVoteRemoved != null) {
        props.onDownVoteRemoved(getCallbackInfoWrap());
      }

      setDownVotes(downVotes - 1);
      setIHaveDownVoted(false);
      setFlowWrap(Constants.FLOW_VIEW);
    }
  }

  function onReplyToClicked() {
    if (getReplyToWrap().userName != null) {
      props.onReplyTo({
        callbackInfo: getCallbackInfoWrap(),
        replyTo: getReplyToWrap()
      });
    }
  }

  function auto_grow(event) {
    event.target.style.height = "5px";
    event.target.style.height = event.target.scrollHeight + "px";
  }

  useEffect(function () {
    validateSubmit();
  }, [uploadResult]);
  useEffect(function () {
    if (!dirty) {
      if (getPreFillWrap() != null) {
        if (getPreFillWrap().text != null) {
          setTextNew(getPreFillWrap().text);
          setTextOriginal(getPreFillWrap().text);
        }

        if (refInputNew.current != null) {
          if (getPreFillWrap().text != null && refInputNew.current != null) {
            refInputNew.current.value = getPreFillWrap().text;
          }
        }

        if (getPreFillWrap().attachment != null) {
          setUploadResult(getPreFillWrap().attachment.object);
          setUploadType(getPreFillWrap().attachment.type);
        } else {
          setUploadResult('');
        }
      }
    }

    if (mode === Constants.MODE_VIEW) {
      if (flow === Constants.FLOW_SHOW_EDIT) {
        setFlowWrap(Constants.FLOW_SHOW_EDIT);
      } else if (flow === Constants.FLOW_CONFIRM_DELETE) {
        setDeleteTimeoutHandler(setTimeout(function () {
          if (flow === Constants.FLOW_CONFIRM_DELETE) {
            setFlowWrap(Constants.FLOW_VIEW);
          }
        }, 5000));
      } else if (flow === Constants.FLOW_CONFIRM_REMOVE_LIKE) {
        setRemoveLikeTimeoutHandler(setTimeout(function () {
          if (flow === Constants.FLOW_CONFIRM_REMOVE_LIKE) {
            setFlowWrap(Constants.FLOW_VIEW);
          }
        }, 5000));
      } else if (flow === Constants.FLOW_CONFIRM_REMOVE_DISLIKE) {
        setRemoveDisLikeTimeoutHandler(setTimeout(function () {
          if (flow === Constants.FLOW_CONFIRM_REMOVE_DISLIKE) {
            setFlowWrap(Constants.FLOW_VIEW);
          }
        }, 5000));
      } else if (flow === Constants.FLOW_CONFIRM_REMOVE_UPVOTE) {
        setRemoveUpVoteTimeoutHandler(setTimeout(function () {
          if (flow === Constants.FLOW_CONFIRM_REMOVE_UPVOTE) {
            setFlowWrap(Constants.FLOW_VIEW);
          }
        }, 5000));
      } else if (flow === Constants.FLOW_CONFIRM_REMOVE_DOWNVOTE) {
        setRemoveDownVoteTimeoutHandler(setTimeout(function () {
          if (flow === Constants.FLOW_CONFIRM_REMOVE_DOWNVOTE) {
            setFlowWrap(Constants.FLOW_VIEW);
          }
        }, 5000));
      } else if (flow === Constants.FLOW_VIEW) {
        clearTimeout(deleteTimeoutHandler);
        clearTimeout(removeLikeTimeoutHandler);
        clearTimeout(removeDisLikeTimeoutHandler);
        clearTimeout(removeUpVoteTimeoutHandler);
        clearTimeout(removeDownVoteTimeoutHandler);
      } else if (flow === Constants.FLOW_INIT) {
        if (refInputNew.current != null) {
          refInputNew.current.focus();
          refInputNew.current.value = textNew;
        }
      } else if (flow === Constants.FLOW_UPLOAD_COMPLETE) {
        if (refInputNew.current != null) {
          refInputNew.current.focus();
          refInputNew.current.value = textNew;
        }
      }
    }

    if (mode === Constants.MODE_EDIT) {
      if (refInputNew.current != null) {
        refInputNew.current.focus();
        refInputNew.current.value = textNew;
      }

      if (showUpload) {
        setFlowWrap(Constants.FLOW_UPLOAD);
      } else if (showEmoji) {
        setFlowWrap(Constants.FLOW_EMOJI_PICKER);
      } else if (showAttachments) {
        setFlowWrap(Constants.FLOW_SHOW_ATTACHMENT);
      } else {
        if (uploadResult != '') {
          setFlowWrap(Constants.FLOW_UPLOAD_COMPLETE);
        } else {
          setFlowWrap(Constants.FLOW_INIT);
        }
      }
    }

    if (mode === Constants.MODE_DELETED) {
      if (flow === Constants.FLOW_DELETED) {
        clearTimeout(deleteTimeoutHandler);
      }
    }
  }, [preFill, flow, refInputNew.current, showEmoji, showAttachments, showUpload, replyTo]);
  useEffect(function () {
    if (mode == Constants.MODE_EDIT) {
      setFlowWrap(Constants.FLOW_INIT);
    }

    if (mode == Constants.MODE_VIEW) {
      setFlowWrap(Constants.FLOW_VIEW);
    }

    if (mode == Constants.MODE_DELETED) {
      setFlowWrap(Constants.FLOW_DELETED);
    }
  }, [mode]);
  useEffect(function () {
    setLikes(props.likes);
  }, [props.likes]);
  useEffect(function () {
    setDisLikes(props.disLikes);
  }, [props.disLikes]);
  useEffect(function () {
    setIHaveLiked(props.iHaveLiked);
  }, [props.iHaveLiked]);
  useEffect(function () {
    setIHaveDisLiked(props.iHaveDisLiked);
  }, [props.iHaveDisLiked]);
  useEffect(function () {
    setUpVotes(props.upVotes);
  }, [props.upVotes]);
  useEffect(function () {
    setDownVotes(props.downVotes);
  }, [props.downVotes]);
  useEffect(function () {
    setIHaveUpVoted(props.iHaveUpVoted);
  }, [props.iHaveUpVoted]);
  useEffect(function () {
    setIHaveDownVoted(props.iHaveDownVoted);
  }, [props.iHaveDownVoted]);
  useEffect(function () {
    setCallbackInfoWrap(props.callbackInfo);
  }, [props.callbackInfo]);
  useEffect(function () {
    setEdited(props.edited);
  }, [props.edited]);
  useEffect(function () {
    setMode(props.mode);
  }, [props.mode]);
  useEffect(function () {
    setReplyToWrap(props.replyTo != null ? props.replyTo : {});
  }, [props.replyTo]);
  useEffect(function () {
    if (props.preFill != null) {
      setPreFillWrap(props.preFill);
    }
  }, [(_props$preFill = props.preFill) === null || _props$preFill === void 0 ? void 0 : _props$preFill.text]);
  return /*#__PURE__*/React.createElement("div", {
    style: {
      position: 'relative'
    }
  }, /*#__PURE__*/React.createElement(Container, {
    className: "w-100 rounded-3 d-flex flex-column px-3 pb-3",
    style: {
      border: 'solid 1px',
      borderColor: props.theme != null ? props.theme.commentViewBorderColor : theme.commentViewBorderColor,
      backgroundColor: props.showEdit != null && props.showEdit || props.showDelete != null && props.showDelete ? props.theme != null ? props.theme.commentViewMyBackgroundColor : theme.commentViewMyBackgroundColor : props.theme != null ? props.theme.commentViewBackgroundColor : theme.commentViewBackgroundColor
    }
  }, /*#__PURE__*/React.createElement("div", {
    className: "d-flex align-items-center justify-content-between mt-3 mb-0 p-0",
    role: "alert"
  }, /*#__PURE__*/React.createElement("div", {
    className: "d-flex align-items-center"
  }, /*#__PURE__*/React.createElement("div", {
    className: "me-2",
    style: {
      width: '40px',
      height: '40px',
      backgroundImage: "url(" + props.user.picture + ")",
      backgroundSize: 'cover',
      backgroundRepeat: 'no-repeat',
      backgroundPosition: 'center',
      borderRadius: '40px'
    }
  }), /*#__PURE__*/React.createElement("div", {
    style: {
      color: props.theme != null ? props.theme.commentViewUserColor : theme.commentViewUserColor
    }
  }, /*#__PURE__*/React.createElement("b", null, props.user.name)), (flow == Constants.FLOW_VIEW || flow == Constants.FLOW_CONFIRM_DELETE || flow == Constants.FLOW_CONFIRM_REMOVE_LIKE || flow == Constants.FLOW_CONFIRM_REMOVE_DISLIKE || flow == Constants.FLOW_CONFIRM_REMOVE_UPVOTE || flow == Constants.FLOW_CONFIRM_REMOVE_DOWNVOTE) && /*#__PURE__*/React.createElement("div", {
    style: {
      color: props.theme != null ? props.theme.commentViewUserColor : theme.commentViewUserColor
    }
  }, /*#__PURE__*/React.createElement(Dot, null)), (flow == Constants.FLOW_INIT || flow == Constants.FLOW_UPLOAD_COMPLETE || flow == Constants.FLOW_SHOW_ATTACHMENT) && props.showCancel != null && props.showCancel && /*#__PURE__*/React.createElement("div", {
    style: {
      color: props.theme != null ? props.theme.commentViewUserColor : theme.commentViewUserColor
    }
  }, /*#__PURE__*/React.createElement(Dot, null)), (flow == Constants.FLOW_INIT || flow == Constants.FLOW_UPLOAD_COMPLETE || flow == Constants.FLOW_SHOW_ATTACHMENT) && props.showCancel != null && props.showCancel && /*#__PURE__*/React.createElement(Button, {
    style: {
      color: props.theme != null ? props.theme.commentViewDecorationColor : theme.commentViewDecorationColor
    },
    className: "button_cancel",
    onClick: function onClick() {
      onCancelClicked();
    },
    variant: "btn btn-outline"
  }, /*#__PURE__*/React.createElement("small", null, "Cancel"))), (flow == Constants.FLOW_VIEW || flow == Constants.FLOW_CONFIRM_DELETE || flow == Constants.FLOW_CONFIRM_REMOVE_LIKE || flow == Constants.FLOW_CONFIRM_REMOVE_DISLIKE || flow == Constants.FLOW_CONFIRM_REMOVE_UPVOTE || flow == Constants.FLOW_CONFIRM_REMOVE_DOWNVOTE) && (props.showEdit != null && props.showEdit || props.showDelete != null && props.showDelete) && /*#__PURE__*/React.createElement("div", {
    className: "button_show_edit d-flex me-2",
    ref: refPopup,
    onClick: function onClick() {
      showEdit(true);
    },
    style: {
      cursor: 'pointer',
      color: props.theme != null ? props.theme.commentViewDecorationColor : theme.commentViewDecorationColor
    }
  }, /*#__PURE__*/React.createElement(ChevronDown, null)), flow == Constants.FLOW_SHOW_EDIT && (props.showEdit != null && props.showEdit || props.showDelete != null && props.showDelete) && /*#__PURE__*/React.createElement("div", {
    className: "button_show_edit d-flex me-2",
    ref: refPopup,
    onClick: function onClick() {
      showEdit(false);
    },
    style: {
      cursor: 'pointer',
      color: props.theme != null ? props.theme.commentViewDecorationColor : theme.commentViewDecorationColor
    }
  }, /*#__PURE__*/React.createElement(ChevronUp, null))), (flow == Constants.FLOW_SHOW_EDIT || flow == Constants.FLOW_CONFIRM_DELETE) && /*#__PURE__*/React.createElement("div", {
    className: "d-flex justify-content-between"
  }, /*#__PURE__*/React.createElement("div", null), /*#__PURE__*/React.createElement("div", null, props.showEdit != null && props.showEdit && /*#__PURE__*/React.createElement(Button, {
    className: "button_edit",
    onClick: function onClick() {
      onEditClicked();
    },
    variant: "btn btn-outline",
    style: {
      color: props.theme != null ? props.theme.commentViewDecorationColor : theme.commentViewDecorationColor
    }
  }, /*#__PURE__*/React.createElement("small", null, /*#__PURE__*/React.createElement("small", null, "Edit"))), props.showDelete != null && props.showDelete && /*#__PURE__*/React.createElement(Button, {
    className: "button_delete ",
    onClick: function onClick() {
      onDeleteClicked();
    },
    variant: "btn btn-outline",
    style: {
      color: props.theme != null ? props.theme.commentViewDecorationColor : theme.commentViewDecorationColor
    }
  }, /*#__PURE__*/React.createElement("small", null, /*#__PURE__*/React.createElement("small", null, "Delete"))))), flow == Constants.FLOW_DELETED && /*#__PURE__*/React.createElement(Container, {
    className: "d-flex flex-column flex-grow-1 p-0"
  }, /*#__PURE__*/React.createElement("div", {
    className: "slash_circle py-2",
    style: {
      color: props.theme != null ? props.theme.commentViewColor : theme.commentViewColor
    }
  }, /*#__PURE__*/React.createElement(SlashCircle, {
    className: "me-2"
  }), Constants.CONTENT_DELETED)), (flow == Constants.FLOW_VIEW || flow == Constants.FLOW_CONFIRM_DELETE || flow == Constants.FLOW_CONFIRM_REMOVE_LIKE || flow == Constants.FLOW_CONFIRM_REMOVE_DISLIKE || flow == Constants.FLOW_CONFIRM_REMOVE_UPVOTE || flow == Constants.FLOW_CONFIRM_REMOVE_DOWNVOTE || flow == Constants.FLOW_SHOW_EDIT) && /*#__PURE__*/React.createElement(Container, {
    className: "d-flex p-0"
  }, props.showVotes && /*#__PURE__*/React.createElement("div", {
    className: "d-flex flex-column justify-content-start align-items-center py-2",
    style: {
      width: '45px',
      marginRight: '7px'
    }
  }, /*#__PURE__*/React.createElement("small", null, iHaveUpVoted && /*#__PURE__*/React.createElement(CaretUpFill, {
    className: "button_upvoted",
    onClick: function onClick() {
      onUpVoteRemoved();
    },
    style: {
      cursor: 'pointer',
      color: props.theme != null ? props.theme.commentViewDecorationHighlightColor : theme.commentViewDecorationHighlightColor
    }
  }), !iHaveUpVoted && /*#__PURE__*/React.createElement(CaretUp, {
    className: "button_upvote",
    onClick: function onClick() {
      onUpVoted();
    },
    style: {
      cursor: 'pointer',
      color: props.theme != null ? props.theme.commentViewDecorationColor : theme.commentViewDecorationColor
    }
  })), /*#__PURE__*/React.createElement("small", null, (iHaveUpVoted || iHaveDownVoted) && /*#__PURE__*/React.createElement("span", {
    style: {
      color: props.theme != null ? props.theme.commentViewDecorationHighlightColor : theme.commentViewDecorationHighlightColor
    }
  }, upVotes - downVotes), !iHaveUpVoted && !iHaveDownVoted && /*#__PURE__*/React.createElement("span", {
    style: {
      color: props.theme != null ? props.theme.commentViewDecorationColor : theme.commentViewDecorationColor
    }
  }, upVotes - downVotes)), /*#__PURE__*/React.createElement("small", null, iHaveDownVoted && /*#__PURE__*/React.createElement(CaretDownFill, {
    className: "button_downvoted",
    onClick: function onClick() {
      onDownVoteRemoved();
    },
    style: {
      cursor: 'pointer',
      color: props.theme != null ? props.theme.commentViewDecorationHighlightColor : theme.commentViewDecorationHighlightColor
    }
  }), !iHaveDownVoted && /*#__PURE__*/React.createElement(CaretDown, {
    className: "button_downvote",
    onClick: function onClick() {
      onDownVoted();
    },
    style: {
      cursor: 'pointer',
      color: props.theme != null ? props.theme.commentViewDecorationColor : theme.commentViewDecorationColor
    }
  }))), /*#__PURE__*/React.createElement(Container, {
    className: "d-flex flex-column flex-grow-1 p-0"
  }, getReplyToWrap().userName != null && /*#__PURE__*/React.createElement("div", {
    className: "p-2 rounded-3 mt-3",
    style: {
      backgroundColor: props.theme.commentViewReplyToBackgroundColor != null ? props.theme.commentViewReplyToBackgroundColor : theme.commentViewReplyToBackgroundColor,
      color: props.theme.commentViewReplyToTitleColor != null ? props.theme.commentViewReplyToTitleColor : theme.commentViewReplyToTitleColor,
      borderLeftWidth: '5px',
      borderLeftColor: props.theme.commentViewReplyToTitleColor != null ? props.theme.commentViewReplyToTitleColor : theme.commentViewReplyToTitleColor,
      borderLeftStyle: 'solid',
      cursor: 'pointer'
    },
    onClick: function onClick() {
      onReplyToClicked();
    }
  }, /*#__PURE__*/React.createElement("div", null, /*#__PURE__*/React.createElement("b", null, getReplyToWrap().userName.substring(0, 40))), /*#__PURE__*/React.createElement("div", null, /*#__PURE__*/React.createElement("small", null, getReplyToWrap().text.substring(0, 40)))), /*#__PURE__*/React.createElement("div", {
    className: "py-3",
    style: {
      color: props.theme.commentViewColor != null ? props.theme.commentViewColor : theme.commentViewColor
    }
  }, textNew), uploadResult.length > 0 && /*#__PURE__*/React.createElement("div", {
    className: "d-flex justify-content-between"
  }, /*#__PURE__*/React.createElement("div", {
    className: "d-flex align-items-center mb-0 px-0 pt-0 pb-2",
    role: "alert",
    style: {
      cursor: 'pointer',
      color: props.theme != null ? props.theme.commentViewColor : theme.commentViewColor
    },
    onClick: function onClick() {
      onClickAttachment();
    }
  }, uploadType == Constants.UPLOAD_TYPE_PDF && /*#__PURE__*/React.createElement(FilePdf, {
    className: "me-2"
  }), uploadType == Constants.UPLOAD_TYPE_IMAGE && /*#__PURE__*/React.createElement(Image, {
    className: "me-2"
  }), uploadType == Constants.UPLOAD_TYPE_VIDEO && /*#__PURE__*/React.createElement(FilePlay, {
    className: "me-2"
  }), /*#__PURE__*/React.createElement("div", null, /*#__PURE__*/React.createElement("small", null, Util.ellipsizeStart(uploadResult, 15))), /*#__PURE__*/React.createElement("div", {
    className: "ms-2"
  }, /*#__PURE__*/React.createElement("small", null, /*#__PURE__*/React.createElement("small", null, /*#__PURE__*/React.createElement(ArrowRight, null), " "))))), /*#__PURE__*/React.createElement("div", {
    className: "d-flex justify-content-end align-items-center text-muted"
  }, /*#__PURE__*/React.createElement(Button, {
    className: "button_reply ps-0 pe-0 py-0 me-2",
    onClick: function onClick() {
      onReplyClicked();
    },
    variant: "btn btn-outline",
    style: {
      color: props.theme != null ? props.theme.commentViewColor : theme.commentViewColor
    }
  }, /*#__PURE__*/React.createElement("small", null, "Reply")), props.showShare && /*#__PURE__*/React.createElement(Dot, {
    style: {
      color: props.theme != null ? props.theme.commentViewColor : theme.commentViewColor
    }
  }), props.showShare && /*#__PURE__*/React.createElement(Button, {
    className: "button_reply ps-0 pe-0 py-0 ms-2 me-2",
    onClick: function onClick() {
      onShareClicked();
    },
    variant: "btn btn-outline",
    style: {
      color: props.theme != null ? props.theme.commentViewColor : theme.commentViewColor
    }
  }, /*#__PURE__*/React.createElement("small", null, "Share"))), (props.showLikes || props.showDisLikes || props.user.timestamp != null) && /*#__PURE__*/React.createElement("div", {
    className: "d-flex justify-content-start align-items-center text-muted mt-2"
  }, /*#__PURE__*/React.createElement("div", {
    className: "d-flex w-100 justify-content-end align-items-center"
  }, props.user.timestamp != null && /*#__PURE__*/React.createElement("div", {
    style: {
      color: props.theme != null ? props.theme.commentViewDecorationColor : theme.commentViewDecorationColor
    },
    className: "me-3"
  }, /*#__PURE__*/React.createElement("small", null, /*#__PURE__*/React.createElement("small", null, Util.timeDifference(new Date().getTime(), parseInt(props.user.timestamp) * 1000)))), edited && /*#__PURE__*/React.createElement(Dot, {
    className: "me-2",
    style: {
      color: props.theme != null ? props.theme.commentViewDecorationColor : theme.commentViewDecorationColor
    }
  }), edited && /*#__PURE__*/React.createElement(Button, {
    className: "button_reply ps-0 pe-0 py-0 me-2",
    disabled: true,
    variant: "btn btn-link",
    style: {
      color: props.theme != null ? props.theme.commentViewDecorationColor : theme.commentViewDecorationColor
    }
  }, /*#__PURE__*/React.createElement("small", null, /*#__PURE__*/React.createElement("small", null, "Edited"))), edited && /*#__PURE__*/React.createElement(Dot, {
    className: "me-2"
  }), props.showLikes && /*#__PURE__*/React.createElement("div", {
    className: "d-flex justify-content-start align-items-center me-4"
  }, /*#__PURE__*/React.createElement("small", null, iHaveLiked && /*#__PURE__*/React.createElement("span", {
    style: {
      color: props.theme != null ? props.theme.commentViewDecorationHighlightColor : theme.commentViewDecorationHighlightColor
    }
  }, /*#__PURE__*/React.createElement(HandThumbsUpFill, {
    className: "button_liked me-2",
    style: {
      marginBottom: '2px',
      cursor: 'pointer'
    },
    onClick: function onClick() {
      onLikeRemoved();
    }
  }), likes), !iHaveLiked && /*#__PURE__*/React.createElement("span", {
    style: {
      color: props.theme != null ? props.theme.commentViewDecorationColor : theme.commentViewDecorationColor
    }
  }, /*#__PURE__*/React.createElement(HandThumbsUp, {
    className: "button_like me-2",
    style: {
      marginBottom: '2px',
      cursor: 'pointer'
    },
    onClick: function onClick() {
      onLiked();
    }
  }), likes))), props.showDisLikes && /*#__PURE__*/React.createElement("div", {
    className: "d-flex justify-content-start align-items-center me-2"
  }, /*#__PURE__*/React.createElement("small", null, iHaveDisLiked && /*#__PURE__*/React.createElement("span", {
    style: {
      color: props.theme != null ? props.theme.commentViewDecorationHighlightColor : theme.commentViewDecorationHighlightColor
    }
  }, /*#__PURE__*/React.createElement(HandThumbsDownFill, {
    className: "button_disliked me-2",
    style: {
      marginBottom: '2px',
      cursor: 'pointer'
    },
    onClick: function onClick() {
      onDisLikeRemoved();
    }
  }), disLikes), !iHaveDisLiked && /*#__PURE__*/React.createElement("span", {
    style: {
      color: props.theme != null ? props.theme.commentViewDecorationColor : theme.commentViewDecorationColor
    }
  }, /*#__PURE__*/React.createElement(HandThumbsDown, {
    className: "button_dislike me-2",
    style: {
      marginBottom: '2px',
      cursor: 'pointer'
    },
    onClick: function onClick() {
      onDisLiked();
    }
  }), disLikes))))), flow === Constants.FLOW_CONFIRM_REMOVE_LIKE && /*#__PURE__*/React.createElement("div", {
    className: "d-flex justify-content-end",
    style: {
      color: props.theme != null ? props.theme.commentViewDecorationColor : theme.commentViewDecorationColor
    }
  }, /*#__PURE__*/React.createElement("div", null, /*#__PURE__*/React.createElement("small", null, /*#__PURE__*/React.createElement("small", null, /*#__PURE__*/React.createElement("small", null, "Press again to remove like"))))), flow === Constants.FLOW_CONFIRM_REMOVE_DISLIKE && /*#__PURE__*/React.createElement("div", {
    className: "d-flex justify-content-end",
    style: {
      color: props.theme != null ? props.theme.commentViewDecorationColor : theme.commentViewDecorationColor
    }
  }, /*#__PURE__*/React.createElement("div", null, /*#__PURE__*/React.createElement("small", null, /*#__PURE__*/React.createElement("small", null, /*#__PURE__*/React.createElement("small", null, "Press again to remove dislike"))))), flow === Constants.FLOW_CONFIRM_DELETE && /*#__PURE__*/React.createElement("div", {
    className: "d-flex justify-content-end",
    style: {
      color: props.theme != null ? props.theme.commentViewDecorationColor : theme.commentViewDecorationColor
    }
  }, /*#__PURE__*/React.createElement("div", null, /*#__PURE__*/React.createElement("small", null, /*#__PURE__*/React.createElement("small", null, /*#__PURE__*/React.createElement("small", null, "Press again to delete"))))), flow === Constants.FLOW_CONFIRM_REMOVE_UPVOTE && /*#__PURE__*/React.createElement("div", {
    className: "d-flex justify-content-start",
    style: {
      color: props.theme != null ? props.theme.commentViewDecorationColor : theme.commentViewDecorationColor
    }
  }, /*#__PURE__*/React.createElement("div", null, /*#__PURE__*/React.createElement("small", null, /*#__PURE__*/React.createElement("small", null, /*#__PURE__*/React.createElement("small", null, "Press again to remove upvote"))))), flow === Constants.FLOW_CONFIRM_REMOVE_DOWNVOTE && /*#__PURE__*/React.createElement("div", {
    className: "d-flex justify-content-start",
    style: {
      color: props.theme != null ? props.theme.commentViewDecorationColor : theme.commentViewDecorationColor
    }
  }, /*#__PURE__*/React.createElement("div", null, /*#__PURE__*/React.createElement("small", null, /*#__PURE__*/React.createElement("small", null, /*#__PURE__*/React.createElement("small", null, "Press again to remove downvote"))))))), (flow === Constants.FLOW_INIT || flow === Constants.FLOW_UPLOAD_COMPLETE || flow === Constants.FLOW_EMOJI_PICKER || flow == Constants.FLOW_SHOW_ATTACHMENT) && /*#__PURE__*/React.createElement(Container, {
    className: "d-flex flex-column flex-grow-1 p-0 my-1"
  }, getReplyToWrap().userName != null && /*#__PURE__*/React.createElement("div", {
    className: "p-2 rounded-3 my-2",
    style: {
      backgroundColor: props.theme.commentViewReplyToBackgroundColor != null ? props.theme.commentViewReplyToBackgroundColor : theme.commentViewReplyToBackgroundColor,
      color: props.theme.commentViewReplyToTitleColor != null ? props.theme.commentViewReplyToTitleColor : theme.commentViewReplyToTitleColor,
      borderLeftWidth: '5px',
      borderLeftColor: props.theme.commentViewReplyToTitleColor != null ? props.theme.commentViewReplyToTitleColor : theme.commentViewReplyToTitleColor,
      borderLeftStyle: 'solid',
      cursor: 'pointer'
    },
    onClick: function onClick() {
      onReplyToClicked();
    }
  }, /*#__PURE__*/React.createElement("div", {
    className: "d-flex justify-content-between align-items-center"
  }, /*#__PURE__*/React.createElement("div", null, getReplyToWrap().userName.substring(0, 40)), props.showReplyToClose != null && props.showReplyToClose && /*#__PURE__*/React.createElement("div", {
    className: "button_replyto_close px-2",
    onClick: function onClick(event) {
      event.stopPropagation();
      onReplyToClosed();
    },
    style: {
      cursor: 'pointer'
    }
  }, /*#__PURE__*/React.createElement(X, null))), /*#__PURE__*/React.createElement("div", null, /*#__PURE__*/React.createElement("small", null, getReplyToWrap().text.substring(0, 40)))), /*#__PURE__*/React.createElement("div", {
    className: "d-flex align-items-end"
  }, (flow === Constants.FLOW_INIT || flow === Constants.FLOW_UPLOAD_COMPLETE || flow === Constants.FLOW_SHOW_ATTACHMENT) && /*#__PURE__*/React.createElement(Button, {
    className: "btn_emoji",
    variant: "btn-outline-secondary me-1",
    onClick: function onClick() {
      openEmojiPicker(true);
    },
    style: {
      color: props.theme != null ? props.theme.commentViewColor : theme.commentViewColor
    }
  }, /*#__PURE__*/React.createElement(EmojiSmile, null)), flow === Constants.FLOW_EMOJI_PICKER && /*#__PURE__*/React.createElement(Button, {
    className: "btn_cancel_emoji",
    variant: "btn-outline-secondary me-1",
    onClick: function onClick() {
      openEmojiPicker(false);
    },
    style: {
      color: props.theme != null ? props.theme.commentViewColor : theme.commentViewColor
    }
  }, /*#__PURE__*/React.createElement(Keyboard, null)), /*#__PURE__*/React.createElement("textarea", {
    ref: refInputNew,
    className: "text_input rounded-3 flex-grow-1 border-0 mt-2 py-1 px-2 me-1",
    style: {
      resize: 'none',
      overflow: 'hidden',
      height: '35px',
      minHeight: '35px'
    },
    onChange: function onChange() {},
    onKeyUp: function onKeyUp(event) {
      onTextAreaKeyUp(event);
    },
    onInput: function onInput(event) {
      auto_grow(event);
    }
  }), (flow === Constants.FLOW_INIT || flow === Constants.FLOW_EMOJI_PICKER) && /*#__PURE__*/React.createElement(Button, {
    className: "btn_paperclip",
    variant: "btn-outline-secondary me-1",
    onClick: function onClick() {
      showAttachment(true);
    },
    style: {
      color: props.theme != null ? props.theme.commentViewColor : theme.commentViewColor
    }
  }, /*#__PURE__*/React.createElement(Paperclip, null)), flow === Constants.FLOW_SHOW_ATTACHMENT && /*#__PURE__*/React.createElement(Button, {
    className: "btn_cancel_paperclip",
    variant: "btn-outline-secondary mx-1",
    onClick: function onClick() {
      showAttachment(false);
    },
    style: {
      color: props.theme != null ? props.theme.commentViewColor : theme.commentViewColor
    }
  }, /*#__PURE__*/React.createElement(ChevronUp, null)), /*#__PURE__*/React.createElement(Button, {
    className: "btn_send",
    variant: "btn btn-secondary",
    onClick: function onClick() {
      submitResult();
    },
    disabled: disableSubmit,
    style: {
      backgroundColor: theme.uploadToS3UploadBackgroundColor,
      color: theme.uploadToS3UploadColor
    }
  }, /*#__PURE__*/React.createElement(Send, null)))), flow === Constants.FLOW_UPLOAD_COMPLETE && /*#__PURE__*/React.createElement(Container, {
    className: "d-flex flex-row justify-content-start align-items-center ps-0 pe-0 pt-2"
  }, /*#__PURE__*/React.createElement("div", {
    className: "d-flex alert alert-secondary align-items-center mb-0 p-0 px-2",
    role: "alert",
    style: {
      backgroundColor: props.theme.commentViewReplyToBackgroundColor != null ? props.theme.commentViewReplyToBackgroundColor : theme.commentViewReplyToBackgroundColor,
      color: props.theme.commentViewReplyToTitleColor != null ? props.theme.commentViewReplyToTitleColor : theme.commentViewReplyToTitleColor
    }
  }, uploadType == Constants.UPLOAD_TYPE_PDF && /*#__PURE__*/React.createElement(FilePdf, {
    className: "me-2",
    style: {
      marginBottom: '2px'
    }
  }), uploadType == Constants.UPLOAD_TYPE_IMAGE && /*#__PURE__*/React.createElement(FileImage, {
    className: "me-2",
    style: {
      marginBottom: '2px'
    }
  }), uploadType == Constants.UPLOAD_TYPE_VIDEO && /*#__PURE__*/React.createElement(FilePlay, {
    className: "me-2",
    style: {
      marginBottom: '2px'
    }
  }), /*#__PURE__*/React.createElement("div", null, Util.ellipsizeStart(uploadResult, 15)), /*#__PURE__*/React.createElement(Button, {
    variant: "btn btn-sm btn-secondary pt-0 pb-0 ms-2 mt-1 mb-1"
  }, /*#__PURE__*/React.createElement(Trash, {
    style: {
      marginBottom: '2px'
    },
    onClick: function onClick() {
      onRemoveUpload();
    }
  })))), flow === Constants.FLOW_EMOJI_PICKER && /*#__PURE__*/React.createElement(Container, {
    className: "d-flex flex-row justify-content-start align-items-start ps-0 pe-0 pt-2"
  }, /*#__PURE__*/React.createElement(Picker, {
    onEmojiClick: function onEmojiClick(event, obj) {
      onEmojiChosen(event, obj);
    },
    pickerStyle: {
      width: '100%'
    }
  })), flow == Constants.FLOW_SHOW_ATTACHMENT && /*#__PURE__*/React.createElement(Container, {
    className: "d-flex flex-row justify-content-center align-items-center ps-0 pe-0 pt-2 pb-2"
  }, /*#__PURE__*/React.createElement("div", {
    style: {
      visibility: 'hidden'
    }
  }), /*#__PURE__*/React.createElement(Button, {
    className: "btn-video",
    variant: "btn-outline-secondary mx-2",
    onClick: function onClick() {
      prepareUpload(Constants.UPLOAD_TYPE_VIDEO);
    },
    style: {
      color: props.theme != null ? props.theme.commentViewColor : theme.commentViewColor
    }
  }, /*#__PURE__*/React.createElement(PlayBtn, null)), /*#__PURE__*/React.createElement(Button, {
    className: "btn-pdf",
    variant: "btn-outline-secondary mx-2",
    onClick: function onClick() {
      prepareUpload(Constants.UPLOAD_TYPE_PDF);
    },
    style: {
      color: props.theme != null ? props.theme.commentViewColor : theme.commentViewColor
    }
  }, /*#__PURE__*/React.createElement(Paperclip, null)), /*#__PURE__*/React.createElement(Button, {
    className: "btn-image",
    variant: "btn-outline-secondary mx-2",
    onClick: function onClick() {
      prepareUpload(Constants.UPLOAD_TYPE_IMAGE);
    },
    style: {
      color: props.theme != null ? props.theme.commentViewColor : theme.commentViewColor
    }
  }, /*#__PURE__*/React.createElement(CardImage, null))), flow === Constants.FLOW_UPLOAD && /*#__PURE__*/React.createElement(Container, {
    className: "d-flex flex-row justify-content-end align-items-center ps-0 pe-0 pt-2 pb-2"
  }, /*#__PURE__*/React.createElement(UploadToS3, {
    bucket: props.bucket,
    awsRegion: props.awsRegion,
    awsKey: props.awsKey,
    awsSecret: props.awsSecret,
    awsMediaConvertEndPoint: props.awsMediaConvertEndPoint,
    type: uploadType,
    mediaConvertRole: props.mediaConvertRole,
    onResult: function onResult(result) {
      onUploadComplete(result);
    },
    showNewUpload: false,
    theme: props.theme != null ? props.theme : theme
  })), flow === Constants.FLOW_UPLOAD && /*#__PURE__*/React.createElement(Container, {
    className: "d-flex flex-row justify-content-end align-items-center ps-0 pe-0 pb-2"
  }, /*#__PURE__*/React.createElement(Button, {
    className: "cancel-uploader",
    variant: "btn btn-sm btn-outline-secondary",
    onClick: cancelUpload,
    style: {
      color: props.theme != null ? props.theme.commentViewColor : theme.commentViewColor
    }
  }, "\u2716"))));
};

export { CommentView };
//# sourceMappingURL=index.modern.js.map
